@@ -3,28 +3,27 @@ import sys
 import threading
 from typing import List, Optional, cast, Callable, Dict
 
-from easyshare import logging
 from easyshare.args import Option, PRESENCE_PARAM, ArgsParseError, ArgType, Args, StrParams, VarArgsSpec
 from easyshare.auth import AuthFactory
+from easyshare.commands.esd import Esd, EsdUsage
 from easyshare.common import APP_VERSION, APP_NAME_SERVER, SERVER_NAME_ALPHABET, easyshare_setup, APP_INFO, \
-    DEFAULT_SERVER_PORT, DEFAULT_DISCOVER_PORT
+    DEFAULT_SERVER_PORT, DEFAULT_DISCOVER_PORT, TRACING_TEXT, VERBOSITY_MAX
 from easyshare.conf import Conf, INT_VAL, STR_VAL, BOOL_VAL, ConfParseError
 from easyshare.esd.common import Sharing
 from easyshare.esd.daemons.api import ApiDaemon
 from easyshare.esd.daemons.discover import DiscoverDaemon
-from easyshare.helps.esd import Esd
 from easyshare.logging import get_logger
 from easyshare.protocol.types import ServerInfoFull
-from easyshare.res.helps import get_command_usage
+from easyshare.res.helps import command_usage
+from easyshare.settings import Settings, set_setting, get_setting
 from easyshare.ssl import get_ssl_context, set_ssl_context
-from easyshare.styling import enable_colors, bold
-from easyshare.tracing import set_tracing_level, TRACING_NONE, TRACING_TEXT
+from easyshare.styling import bold
 from easyshare.utils import terminate, abort
-from easyshare.utils.env import are_colors_supported
+from easyshare.utils.env import is_styling_supported, is_stdout_terminal
 from easyshare.utils.json import j
 from easyshare.utils.net import is_valid_port, get_primary_ip
 from easyshare.utils.ssl import create_server_ssl_context
-from easyshare.utils.str import satisfychars, tf, keepchars
+from easyshare.utils.str import tf, keepchars
 
 # if __name__ == "__main__":
 # Call it now before get_logger for enable colors properly
@@ -87,40 +86,24 @@ class SharingArgs(VarArgsSpec):
 
     def continue_parsing_hook(self) -> Optional[Callable[[str, ArgType, int, Args, List[str]], bool]]:
         # Usually there is no need to do this kind of check for arguments parsing
-        # but in this way we can break the parsing if we found an unknown option
-        # which enables tricks such as not parsing if a -s (esd option) is found,
-        # therefore allows a chain of -s sharing -s sharing ...
+        # but in this way we can break the parsing if we found a -s while we
+        # have already handled a sharing definition; this allows a chain
+        # of -s sharing -s sharing ...
         def continue_parsing_func(argname, argtype, idx, args, positionals):
-            nonlocal self
-            if argtype == ArgType.OPTION and argname in SharingArgs.SHARING:
-                if SharingArgs.get_sharing_params(args):
-                    log.d("Breaking -s chain, sharing params already found")
-                    return False # break the chain
-                log.d("Found the first -s, the next one will break the chain")
+            # log.d(f"continue_parsing_func of ({argname}, {argtype}, "
+            #       f"{idx}, args={args}, positionals={positionals})")
+
+            if (argtype == ArgType.OPTION and argname in SharingArgs.SHARING) and \
+                (positionals or args.get_option_param(SharingArgs.SHARING)):
+                log.d("Breaking -s chain, sharing definition already found")
+                return False # break the chain
+
+            log.d("First sharing definition, the next one will break the chain")
 
             return True
 
         return continue_parsing_func
 
-    @staticmethod
-    def get_sharing_params(args: Args) -> Optional[List[str]]:
-        if not args:
-            log.w("Can't provide sharings params, args not parsed yet?")
-            return None # not parsed yet
-
-        sharings_params = args.get_option_params(SharingArgs.SHARING) # -s params
-        if sharings_params:
-            log.d("Providing sharing params from -s option")
-            return sharings_params
-
-        sharings_params = args.get_positionals() # positionals, without -s
-        if sharings_params:
-            log.d("Providing sharing params from positionals")
-            return sharings_params
-
-        return None
-
-
 class EsdConfKeys:
     """ Keys of the configuration file of esd (-c config)"""
     G_NAME = "name"
@@ -168,31 +151,33 @@ ESD_CONF_SPEC = {
 
 # ==================================================================
 
-def main():
-    # Already called
-    # easyshare_setup()
+running_sync = threading.Semaphore(0)
+is_running_event = threading.Event()
 
-    if len(sys.argv) <= 1:
-        _print_usage_and_quit()
+def main():
+    start(sys.argv[1:])
+
+def start(args):
+    # Already called: easyshare_setup()
 
     # Parse arguments
     g_args = None
 
     try:
-        g_args = Esd().parse(sys.argv[1:])
+        g_args = Esd().parse(args)
     except ArgsParseError as err:
-        log.exception("Exception occurred while parsing args")
-        abort("Parse of global arguments failed: {}".format(str(err)))
+        log.eexception("Exception occurred while parsing args")
+        abort("parse of global arguments failed: {}".format(str(err)))
 
     # Eventually set verbosity before anything else
     # so that the rest of the startup (config parsing, ...)
     # can be logged
     if g_args.has_option(Esd.VERBOSE):
-        log.set_verbosity(g_args.get_option_param(Esd.VERBOSE,
-                                                  default=logging.VERBOSITY_MAX))
+        set_setting(Settings.VERBOSITY,
+                    g_args.get_option_param(Esd.VERBOSE, default=VERBOSITY_MAX))
 
     log.i("{} v. {}".format(APP_NAME_SERVER, APP_VERSION))
-    log.i("Starting with arguments\n%s", g_args)
+    log.i(f"Starting with arguments\n{g_args}")
 
     # Help?
     if Esd.HELP in g_args:
@@ -203,9 +188,9 @@ def main():
         terminate(APP_INFO)
 
     # Default values
-    verbosity = logging.VERBOSITY_NONE
-    tracing = TRACING_NONE
-    no_colors = False
+    verbosity = get_setting(Settings.VERBOSITY)
+    tracing = get_setting(Settings.TRACING)
+    colors = get_setting(Settings.COLORS)
 
     server_name = socket.gethostname()
     server_address = None
@@ -223,10 +208,10 @@ def main():
     # sharings with the same name will keep only the last one
     sharings: Dict[str, Sharing] = {}
 
-    def add_sharing(path: str, name: str, readonly: bool):
+    def add_sharing(path: str, name: str, readonly: bool) -> bool:
         if not path:
-            log.w("Invalid path for sharing '%s'; skipping it", name)
-            return
+            log.w(f"Invalid path for sharing '{name}'; skipping it")
+            return False
 
         sh = Sharing.create(
             name=name,
@@ -236,14 +221,20 @@ def main():
 
         if not sh:
             log.w("Invalid or incomplete sharing config; skipping it")
-            return
+            return False
 
         # Use sh.name instead of name so that we will use the autogenerated one
         # if not provided by the user
-        log.d("Adding sharing for name = %s", sh.name)
+        log.d(f"Adding sharing for name = {sh.name}")
         sharings[sh.name] = sh
 
-        log.d("Currently added sharings: \n%s", j([sh._info_internal() for sh in sharings.values()]))
+        log.d(f"Currently added sharings: \n{j([sh._info_internal() for sh in sharings.values()])}")
+
+        return True
+
+    def add_sharing_or_die(path: str, name: str, readonly: bool):
+        if not add_sharing(path=path, name=name, readonly=readonly):
+            abort(f"failed to create sharing at '{path}'")
 
     # Read config file
 
@@ -257,13 +248,13 @@ def main():
                 comment_prefixes=["#", ";"]
             )
         except ConfParseError as err:
-            log.exception("Exception occurred while parsing conf")
-            abort(f"Parse of config file failed: {err}")
+            log.eexception("Exception occurred while parsing conf")
+            abort(f"parse of config file failed: {err}")
 
         if cfg:
             _, global_section = cfg.global_section()
 
-            log.i("Config file parsed successfully:\n%s", cfg)
+            log.i(f"Config file parsed successfully:\n{cfg}")
 
             # Config's global settings
 
@@ -313,7 +304,7 @@ def main():
 
             no_colors = global_section.get(
                 EsdConfKeys.G_NO_COLOR,
-                no_colors
+                not colors
             )
 
             tracing = global_section.get(
@@ -332,7 +323,7 @@ def main():
                 s_path = s_settings.get(EsdConfKeys.S_PATH)
                 s_readonly = s_settings.get(EsdConfKeys.S_READONLY, False)
 
-                add_sharing(path=s_path, name=s_name, readonly=s_readonly)
+                add_sharing_or_die(path=s_path, name=s_name, readonly=s_readonly)
 
     # Args from command line: eventually overwrite config settings
 
@@ -389,7 +380,7 @@ def main():
 
     # Colors
     if g_args.has_option(Esd.NO_COLOR):
-        no_colors = True
+        colors = False
 
     # Packet tracing
     if g_args.has_option(Esd.TRACE):
@@ -406,46 +397,42 @@ def main():
         # if not specified the default is DEBUG
         verbosity = g_args.get_option_param(
             Esd.VERBOSE,
-            default=logging.VERBOSITY_MAX
+            default=VERBOSITY_MAX
         )
 
     # Logging/Tracing/UI setup
-    log.d("Colors: %s", not no_colors)
-    log.d("Tracing: %s", tracing)
-    log.d("Verbosity: %s", verbosity)
+    log.d(f"Colors: {colors}")
+    log.d(f"Tracing: {tracing}")
+    log.d(f"Verbosity: {verbosity}")
 
-    enable_colors(are_colors_supported() and not no_colors)
-
-    set_tracing_level(tracing)
-
-    # TODO: doesn't work with -c for some reason
-    if verbosity:
-        log.set_verbosity(verbosity)
+    if colors and not is_stdout_terminal():
+        log.w("Disabling colors since detected non-terminal output file")
+        colors = False
 
+    set_setting(Settings.COLORS, is_styling_supported() and colors)
+    set_setting(Settings.TRACING, tracing)
+    set_setting(Settings.VERBOSITY, verbosity)
 
     # Parse sharing arguments
+    # We have to be careful since we could find the name/path either
+    # in positionals or in the params of "-s"
     # This might come either from
     # 1. Positional arguments (no -s specified), only a sharing can be provided
     # 2. Params of a -s option (multiple sharings can be provided)
 
-    s_unparsed = g_args.get_unparsed_args()
+    s_unparsed = g_args.get_unparsed_args(default=[])
 
-    while s_unparsed:
-        log.d("Found %d unparsed args: considering those sharing args", len(s_unparsed))
-
-        s_args = None
-
-        try:
+    try:
+        while True:
             s_args = SharingArgs().parse(s_unparsed)
-        except ArgsParseError as err:
-            log.exception("Exception occurred while parsing args")
-            abort("Parse of sharing arguments failed: {}".format(str(err)))
+            log.d(f"s_args: {s_args}")
+            sharing_params = \
+                s_args.get_option_params(SharingArgs.SHARING) or \
+                s_args.get_positionals()
+            log.d(f"sharing_params: {sharing_params}")
 
-        # We have to be careful since we could find the name/path either
-        # in positionals or in the params of "-s"
-        sharing_params = SharingArgs.get_sharing_params(s_args)
-
-        if sharing_params:
+            if not sharing_params:
+                break
 
             add_sharing(
                 path=sharing_params[0],
@@ -453,12 +440,13 @@ def main():
                 readonly=s_args.get_option_param(SharingArgs.READ_ONLY)
             )
 
-            s_unparsed = s_args.get_unparsed_args() # eventually other -s definitions
-        else:
-            # else - break the chain (discard eventual junk in unparsed args)
-            log.w("No sharing params, discarding trailing junk: %s", s_args.get_unparsed_args())
-
+            # Parse the unparsed args (other -s sharing definitions)
+            s_unparsed = s_args.get_unparsed_args(default=[])
+            log.d(f"s_unparsed: {s_unparsed}")
 
+    except ArgsParseError as err:
+        log.eexception("Exception occurred while parsing args")
+        abort("Parse of sharing arguments failed: {}".format(str(err)))
 
     g_args.get_option_params(Esd.SHARING)
 
@@ -467,18 +455,18 @@ def main():
     # - server name
     server_name = keepchars(server_name, SERVER_NAME_ALPHABET)
     if not server_name:
-        abort("Invalid server name")
+        abort("invalid server name")
 
     # - ports
     for p in [server_port, server_discover_port]:
         # -1 is a special value for disable discovery
         if p and not is_valid_port(p) and p != -1:
-            abort("Invalid port number {}".format(p))
+            abort("invalid port number {}".format(p))
 
     # - is a useful server?
     if not sharings and not server_rexec:
         log.e("No sharings found, and rexec disabled; nothing to do")
-        _print_usage_and_quit()
+        abort("provide at least one valid sharing")
 
     if not sharings:
         log.w("No sharings found, it will be an empty esd")
@@ -490,8 +478,8 @@ def main():
     if server_ssl_enabled:
         if server_ssl_cert and server_ssl_privkey:
             log.i("Creating SSL context")
-            log.i("SSL cert path: %s", server_ssl_cert)
-            log.i("SSL privkey path: %s", server_ssl_privkey)
+            log.i(f"SSL cert path: {server_ssl_cert}")
+            log.i(f"SSL privkey path: {server_ssl_privkey}")
             ssl_context = create_server_ssl_context(
                 cert=server_ssl_cert, privkey=server_ssl_privkey)
         else:
@@ -509,11 +497,11 @@ def main():
     # Auth
     auth = AuthFactory.parse(server_password)
 
-    log.i("Required server name: %s", server_name)
-    log.i("Required server address: %s", server_address)
-    log.i("Required server port: %s", str(server_port))
-    log.i("Required server discover port: %s", str(server_discover_port))
-    log.i("Required auth: %s", auth.algo_type())
+    log.i(f"Required server name: {server_name}")
+    log.i(f"Required server address: {server_address}")
+    log.i(f"Required server port: {server_port}")
+    log.i(f"Required server discover port: {server_discover_port}")
+    log.i(f"Required auth: {auth.algo_type()}")
 
 
     # Compute real name/port/discover port
@@ -548,10 +536,10 @@ def main():
     if is_valid_port(server_discover_port):
         discover_d = DiscoverDaemon(port=server_discover_port, trace=True, server_info=server_info_full)
 
-    log.i("ApiDaemon started at %s:%d", api_d.address(), api_d.port())
+    log.i(f"ApiDaemon started at {api_d.address()}:{api_d.port()}")
 
     if discover_d:
-        log.i("DiscoverDaemon started at %s:%d", discover_d.address(), discover_d.port())
+        log.i(f"DiscoverDaemon started at {discover_d.address()}:{discover_d.port()}")
 
     if not sharings:
         sharings_str = "NONE"
@@ -614,27 +602,37 @@ Version:            {APP_VERSION}
 
         log.i("Ready to handle requests")
 
-        if th_discover:
-            th_discover.join()
-        th_api.join()
+        is_running_event.set()
+        running_sync.acquire()
+        log.d("Semaphore acquired; quitting")
 
     except KeyboardInterrupt:
         log.d("CTRL+C detected; quitting")
-        # Formally not a clean quit of the threads, but who cares we are exiting...
+
+    # log.i("Killing daemons")
+    # if discover_d:
+    #     discover_d.kill()
+    # api_d.kill()
 
     print("\n" + bold("DONE"))
 
+# Actually called just for internal purposed (tests)
+# The standard way to stop the server is CTRL+C
+def stop():
+    log.d("STOP request")
+    running_sync.release()
+
+def wait_until_start():
+    is_running_event.wait()
+    log.d("EVENT: ESD started")
 
 def _print_usage_and_quit():
     """ Prints the esd usage and exit """
-    esd_usage = get_command_usage(Esd.name())
-
-    if not esd_usage:
-        # Something went wrong with the dynamic loading of the usage
-        abort(f"Can't provide usage of '{Esd.name()}'")
-
-    terminate(esd_usage)
+    command_usage(EsdUsage.helpname())
+    terminate()
 
 
 if __name__ == "__main__":
+    if len(sys.argv) <= 1:
+        _print_usage_and_quit()
     main()
\ No newline at end of file
