@@ -1,11 +1,8 @@
 import time
 
-from easyshare.consts import ansi
-from easyshare.styling import enable_colors
 from easyshare.utils.env import terminal_size
 from easyshare.utils.progress import Progressor
 
-
 class SimpleProgressor(Progressor):
 
     def _progress_string(self, progress_ratio: float, t: int):
@@ -18,7 +15,7 @@ class SimpleProgressor(Progressor):
 
 
 if __name__ == "__main__":
-    enable_colors()
+    # enable_styling()
     with SimpleProgressor(10) as pbar:
         for i in range(1, 10):
             pbar.update(i)
